/****************************************************************************
 *
 *   File: configure_expr.cc:
 *   
 *   Author: Mark Stephenson
 *   Date: Tue Jun  3 21:03:26 2003
 *
 *   Function: This subclass of genetic expression exists solely to
 *             create configuration files.  If the configuration file
 *             does not exist when the library is initialized, the
 *             initialization procedure sets the current genetic_expr
 *             to be a configure_expr.  Thus, when evaluate is called,
 *             it will simply create the configuration file and halt.
 *
 ***************************************************************************/

#include "configure_expr.h"


configure_expr::configure_expr (const string& conf_fname) :
  genetic_expr (NULL, NULL, 0),
  m_conf_fname(conf_fname)
{
}


lval
configure_expr::evaluate (double dbls[], 
			  int32 bools[],
			  uint32 num_dbls, 
			  uint32 num_bools,
			  double sample_rate)
{
  ofstream conf_file(m_conf_fname.c_str());

  // Write out the configuration of the evaluation clause.
  finch_claim (conf_file, "Could not open configuration file.");
  conf_file << num_dbls << endl;
  conf_file << num_bools << endl;

  // Close the file and exit.
  conf_file.close ();
  exit (0);

  return m_lval;
}
