/****************************************************************************
 *
 *   File: configure_expr.h:
 *   
 *   Author: Mark Stephenson
 *   Date: Tue Jun  3 20:49:26 2003
 *
 *   Function: This subclass of genetic expression exists solely to
 *             create configuration files.  If the configuration file
 *             does not exist when the library is initialized, the
 *             initialization procedure sets the current genetic_expr
 *             to be a configure_expr.  Thus, when evaluate is called,
 *             it will simply create the configuration file and halt.
 *
 ***************************************************************************/


#ifndef __CONFIGURE_EXPR_H__
#define __CONFIGURE_EXPR_H__

#include "genetic_expr.h"
#include <string>

class configure_expr : public genetic_expr
{
 public:

  configure_expr (const string& conf_fname);

  lval evaluate (double[], int32[], uint32, uint32, double = 0.0);
  lval evaluate (double[], int32[], bool b) { assert (0); return m_lval; }

  genetic_expr *clone () { assert(0); return NULL; }

 protected:

  uint32 typing () const { assert(0); return DOUBLE; }
  char  *opcode () const { assert(0); return NULL; }

  lval m_lval; // To keep the compiler from complaining.
  string m_conf_fname;
};


#endif
