/************************************************************************
 *
 *   File: double_expr.h:
 *   
 *   Author: Mark Stephenson
 *   Date: Tue Aug  6 12:12:45 2002
 *
 *   Function: Expressions that return a double precision value.
 *
 ************************************************************************/

#ifndef __DOUBLE_EXPR_H__
#define __DOUBLE_EXPR_H__


#include "genetic_expr.h"


///////////////////////////////////////////
///////// arithmetic expression ///////////

class double_expr : public genetic_expr
{
 protected:
  double_expr (genetic_expr_bnf*, genetic_expr*, uint32);
  uint32 typing () const  { return DOUBLE; }
};

///////////////////////////////////////////
////////////// expr + expr ////////////////

class expr_add : public double_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, uint32, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  ~expr_add () {};
  
protected:
  expr_add (genetic_expr_bnf*, genetic_expr*);
  char* opcode () const  { return k_expr_add; }
};


///////////////////////////////////////////
////////////// expr - expr ////////////////

class expr_sub : public double_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, uint32, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  ~expr_sub () {};
  
protected:
  expr_sub (genetic_expr_bnf*, genetic_expr*);
  char* opcode () const  { return k_expr_sub; }
};


///////////////////////////////////////////
////////////// expr * expr ////////////////

class expr_mul : public double_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, uint32, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  ~expr_mul () {};
  
protected:
  expr_mul (genetic_expr_bnf*, genetic_expr*);
  char* opcode () const  { return k_expr_mul; }
};


///////////////////////////////////////////
////////////// expr / expr ////////////////

class expr_div : public double_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, uint32, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  ~expr_div () {};
  
protected:
  expr_div (genetic_expr_bnf*, genetic_expr*);
  char* opcode () const  { return k_expr_div; }
};


///////////////////////////////////////////
//////////////  sqrt(expr) ////////////////

class expr_sqrt : public double_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, uint32, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  ~expr_sqrt () {};
  
protected:
  expr_sqrt (genetic_expr_bnf*, genetic_expr*);
  char* opcode () const  { return k_expr_sqrt; }
};


///////////////////////////////////////////
////////// expr ? expr : expr /////////////

class expr_ternary : public double_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, uint32, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  ~expr_ternary () {};
  
protected:
  expr_ternary (genetic_expr_bnf*, genetic_expr*);
  char* opcode () const    { return k_expr_tern; }
};


///////////////////////////////////////////
//////// expr ? expr*expr : expr //////////

class expr_cmul : public double_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, uint32, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  ~expr_cmul () {};
  
protected:
  expr_cmul (genetic_expr_bnf*, genetic_expr*);
  char* opcode () const    { return k_expr_cmul; }
};


///////////////////////////////////////////
/////////// double constant ///////////////

class expr_dbl_const : public double_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*,
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr *clone ();
  void print (ostream&, bool = false) const;
  ~expr_dbl_const () {}

 protected:
  expr_dbl_const (genetic_expr_bnf*, genetic_expr*, double k);
  char* opcode () const      { return k_expr_dbl_const; }
  void  mutate_node ();
  bool  is_constant () const { return true; }

  double gaussian (double, double) const;
  
 private:
  double m_const;
};


///////////////////////////////////////////
/////////// double argument ///////////////

class expr_dbl_arg : public double_expr
{
public:
  static genetic_expr* random (genetic_expr_bnf*, genetic_expr*);
  static genetic_expr* parse_expr (genetic_expr_bnf*, 
				   genetic_expr*, genetic_lexer*);
  lval evaluate (double[], int32[], bool);
  genetic_expr* clone ();
  void print (ostream&, bool = false) const;
  ~expr_dbl_arg () {}

 protected:
  expr_dbl_arg (genetic_expr_bnf*, genetic_expr*, uint32 k);
  char* opcode () const    { return k_expr_dbl_arg; }
  void  mutate_node ();

 private:
  uint32 m_argnum;
};

#endif
