/****************************************************************************
 *
 *   File: finch_interface.h:
 *   
 *   Author: Mark Stephenson
 *   Date: Tue Jun  3 17:13:37 2003
 *
 *   Function: This file contains the interface to the finch library.
 *
 ***************************************************************************/

#ifndef __FINCH_INTERFACE_H__
#define __FINCH_INTERFACE_H__

#ifdef __cplusplus
extern "C" {
#endif

///////////////////////////////////////////////////////////////
//
// Initialize the finch library.  This function should
// only be called once-- when your pass initializes.
// 
///////////////////////////////////////////////////////////////
void   FINCH_initialize_lib ();

///////////////////////////////////////////////////////////////
//
// Evaluate the current priority function with the arguments
// that are passed in.
//
///////////////////////////////////////////////////////////////
double FINCH_evaluate_real_expr   (unsigned num_dbls, unsigned num_bools, ...);
int    FINCH_evaluate_binary_expr (unsigned num_dbls, unsigned num_bools, ...);
double FINCH_evaluate_real_array  (unsigned num_dbls, unsigned num_bools,
                                   double *dbls, int *bools);
int    FINCH_evaluate_binary_array(unsigned num_dbls, unsigned num_bools,
                                   double *dbls, int *bools);
  


///////////////////////////////////////////////////////////////
//
// Delete any dynamically allocated memory that we have
// created.
//
///////////////////////////////////////////////////////////////
void   FINCH_finalize_lib ();


///////////////////////////////////////////////////////////////
//
// Dumps features to the specified file.
//
///////////////////////////////////////////////////////////////
void   FINCH_dump_features (char *filename, 
			    unsigned num_dbls, unsigned num_bools,
			    double *dbls, int *bools);

#ifdef __cplusplus
}
#endif

#endif

