/****************************************************************************
 *
 *   File: benchmark_set.cc:
 *   
 *   Author: Mark Stephenson
 *   Date: Wed Feb 12 15:24:52 2003
 *
 *   Function: The set of benchmarks that the harness will consider.
 *
 ***************************************************************************/


#include "benchmark_set.h"
#include "genetic_expr.h"
#include <algorithm>
#include <stdio.h>

extern int invoke_train_spec_parser (FILE*, benchmark_vector*);


benchmark_set::benchmark_set (char *benchmarks_fname)
{
  FILE *benchmark_spec_file;

  benchmark_spec_file = fopen (benchmarks_fname, "r");
  finch_claim (benchmark_spec_file, 
	       "Could not open benchmark specification file.");

  // This is a lex and yacc defined parser that reads the specification
  // into the vector of benchmark_info pointers.
  invoke_train_spec_parser (benchmark_spec_file, &m_benches);

  // Expand the variable definitions so the commands are
  // ready to execute.
  for (unsigned i = 0; i < this->num_benches(); i++)
    m_benches[i]->expand_variable_definitions ();

  fclose (benchmark_spec_file);
}


void
benchmark_set::sort ()
{
  ::sort (m_benches.begin(), m_benches.end(), bench_lt());
}


void
benchmark_set::summarize (ostream& os)
{
  os << "DSS Rank : BID : Optimizability : Recency\n";
  for (unsigned i = 0; i < this->num_benches(); i++)
  {
    os << i << " : " << m_benches[i]->bid() << " : ";
    os << m_benches[i]->optimizability();
    os << " : " << m_benches[i]->recency() << endl;
  }
}


void
benchmark_set::print (ostream& os)
{
  for (uint32 j = 0; j < this->num_benches(); j++)
  {
    for (uint32 i = 0; i < this->num_benches(); i++)
    {
      if (m_benches[i]->bid() == (int32)j)
      {
	os.setf (ios::fixed, ios::floatfield);
	os.precision (4);
	os << "newbench {\n";
	print_commands (os, "setup", m_benches[i]->get_setup_commands());
	print_commands (os, "profile", m_benches[i]->get_profile_commands());
	print_commands (os, "compile", m_benches[i]->get_compile_commands());
	print_commands (os, "run", m_benches[i]->get_run_commands());
	print_commands (os, "check", m_benches[i]->get_check_commands());
	print_commands (os, "clean", m_benches[i]->get_clean_commands());
	os << "optimizability = " << m_benches[i]->optimizability() << ";\n";
	os << "recency = " << m_benches[i]->recency() << ";\n";
	os << "}\n\n";
	break;
      }
    }
  }
}


void
benchmark_set::print_commands (ostream& os, char *name, command_list *list)
{
  command_list *beg = list;
  if (beg != NULL)
  {
    os << name << " = ";
    while (beg != NULL) 
    {
      os << "\"" << beg->entry << "\"";
      if ((beg = beg->next))
	os << ", ";
    }
    os << ";\n";
  }
}

void
benchmark_set::increment_recencies ()
{
  for (unsigned i = 0; i < this->num_benches(); i++) 
  {
    m_benches[i]->increment_recency ();
  }
}
